@@ -59,7 +59,7 @@ sub basesetup ($$) {
 	# This will require root privileges.  However, this should
 	# only get run at initial chroot setup time.
 	my $groupfile = $session->get('Location') . "/etc/group";
-	system '/bin/sh', '-c', "getent group sbuild >> $groupfile";
+	system("getent group sbuild >> $groupfile");
 	if ($?) {
 	    print STDERR "E: Failed to create group sbuild\n";
 	    return $?
