@@ -130,27 +130,32 @@ sub setup ($) {
 	    VARNAME => 'chroot',
 	    GROUP => 'Chroot options',
 	    DEFAULT => undef,
-	    HELP => 'Default chroot (defaults to distribution[-arch][-sbuild])'
+	    HELP => 'Default chroot (defaults to distribution[-arch][-sbuild])',
+	    CLI_OPTIONS => ['-c', '--chroot']
 	},
 	'BUILD_ARCH_ALL'			=> {
 	    TYPE => 'BOOL',
 	    VARNAME => 'build_arch_all',
 	    GROUP => 'Build options',
 	    DEFAULT => 0,
-	    HELP => 'Build architecture: all packages by default. Set by the --arch-all and --no-arch-all command line options.'
+	    HELP => 'Build architecture: all packages by default.',
+	    CLI_OPTIONS => ['--arch-all', '--no-arch-all']
 	},
 	'BUILD_ARCH_ANY'			=> {
 	    TYPE => 'BOOL',
 	    VARNAME => 'build_arch_any',
 	    GROUP => 'Build options',
 	    DEFAULT => 1,
-	    HELP => 'Build architecture: any packages by default. Set by the --arch-any and --no-arch-any command line options.'
+	    HELP => 'Build architecture: any packages by default.',
+	    CLI_OPTIONS => ['--arch-any', '--no-arch-any']
 	},
 	'NOLOG'					=> {
 	    TYPE => 'BOOL',
-	    GROUP => '__INTERNAL',
+	    VARNAME => 'nolog',
+	    GROUP => 'Logging options',
 	    DEFAULT => 0,
-	    HELP => 'Disable use of log file'
+	    HELP => 'Disable use of log file',
+	    CLI_OPTIONS => ['-n', '--nolog']
 	},
 	'SUDO'					=> {
 	    TYPE => 'STRING',
@@ -220,6 +225,39 @@ sub setup ($) {
 	    DEFAULT => ['-q'],
 	    HELP => 'Additional command-line options for schroot'
 	},
+	'ADT_VIRT_SERVER'			=> {
+	    TYPE => 'STRING',
+	    VARNAME => 'adt_virt_server',
+	    GROUP => 'Programs',
+	    CHECK => sub {
+		my $conf = shift;
+		my $entry = shift;
+		my $key = $entry->{'NAME'};
+		my $program = $conf->get($key);
+
+		# if the adt virtualization server name is only letters a-z
+		# then it is missing the adt-virt- prefix
+		if ($program =~ /^[a-z]+$/) {
+		    $conf->set($key, "adt-virt-$program");
+		}
+
+		# Only validate if needed.
+		if ($conf->get('CHROOT_MODE') eq 'adt') {
+		    $validate_program->($conf, $entry);
+		}
+	    },
+	    DEFAULT => 'adt-virt-schroot',
+	    HELP => 'Path to adt-virt-* binary, selecting the virtualization server.',
+	    CLI_OPTIONS => ['--adt-virt-server']
+	},
+	'ADT_VIRT_SERVER_OPTIONS'			=> {
+	    TYPE => 'ARRAY:STRING',
+	    VARNAME => 'adt_virt_server_options',
+	    GROUP => 'Programs',
+	    DEFAULT => [],
+	    HELP => 'Additional command-line options for adt-virt-*',
+	    CLI_OPTIONS => ['--adt-virt-server-opt', '--adt-virt-server-opts']
+	},
 	'FAKEROOT'				=> {
 	    TYPE => 'STRING',
 	    VARNAME => 'fakeroot',
@@ -278,9 +316,11 @@ sub setup ($) {
 	},
 	'DPKG_BUILDPACKAGE_USER_OPTIONS'	=> {
 	    TYPE => 'ARRAY:STRING',
-	    GROUP => '__INTERNAL',
+	    VARNAME => 'dpkg_buildpackage_user_options',
+	    GROUP => 'Programs',
 	    DEFAULT => [],
-	    HELP => 'Additional command-line options for dpkg-buildpackage.  Not settable in config.'
+	    HELP => 'Additional command-line options for dpkg-buildpackage.',
+	    CLI_OPTIONS => ['--debbuildopt', '--debbuildopts', '--jobs']
 	},
 	'DPKG_SOURCE'				=> {
 	    TYPE => 'STRING',
@@ -295,7 +335,8 @@ sub setup ($) {
 	    VARNAME => 'dpkg_source_opts',
 	    GROUP => 'Programs',
 	    DEFAULT => [],
-	    HELP => 'Additional command-line options for dpkg-source'
+	    HELP => 'Additional command-line options for dpkg-source',
+	    CLI_OPTIONS => ['--dpkg-source-opt', '--dpkg-source-opts']
 	},
 	'DCMD'					=> {
 	    TYPE => 'STRING',
@@ -318,7 +359,8 @@ sub setup ($) {
 	    VARNAME => 'stats_dir',
 	    GROUP => 'Statistics',
 	    DEFAULT => "$HOME/stats",
-	    HELP => 'Directory for writing build statistics to'
+	    HELP => 'Directory for writing build statistics to',
+	    CLI_OPTIONS => ['--stats-dir']
 	},
 	'PACKAGE_CHECKLIST'			=> {
 	    TYPE => 'STRING',
@@ -448,7 +490,8 @@ sub setup ($) {
 		return $retval;
 	    },
 	    DEFAULT => "",
-	    HELP => 'email address to mail build logs to'
+	    HELP => 'email address to mail build logs to',
+	    CLI_OPTIONS => ['--mail-log-to']
 	},
 	'MAILTO_FORCED_BY_CLI'			=> {
 	    TYPE => 'BOOL',
@@ -467,7 +510,8 @@ sub setup ($) {
 	    VARNAME => 'mailfrom',
 	    GROUP => 'Logging options',
 	    DEFAULT => "Source Builder <sbuild>",
-	    HELP => 'email address set in the From line of build logs'
+	    HELP => 'email address set in the From line of build logs',
+	    CLI_OPTIONS => ['--mailfrom']
 	},
 	'COMPRESS_BUILD_LOG_MAILS'              => {
 	    TYPE => 'BOOL',
@@ -498,7 +542,8 @@ sub setup ($) {
 			     qw(always successful never));
 	    },
 	    DEFAULT => 'always',
-	    HELP => 'When to purge the build dependencies after a build; possible values are "never", "successful", and "always"'
+	    HELP => 'When to purge the build dependencies after a build; possible values are "never", "successful", and "always"',
+	    CLI_OPTIONS => ['-p', '--purge', '--purge-deps']
 	},
 	'PURGE_BUILD_DIRECTORY'			=> {
 	    TYPE => 'STRING',
@@ -515,7 +560,8 @@ sub setup ($) {
 			     qw(always successful never));
 	    },
 	    DEFAULT => 'always',
-	    HELP => 'When to purge the build directory after a build; possible values are "never", "successful", and "always"'
+	    HELP => 'When to purge the build directory after a build; possible values are "never", "successful", and "always"',
+	    CLI_OPTIONS => ['-p', '--purge', '--purge-build']
 	},
 	'PURGE_SESSION'			=> {
 	    TYPE => 'STRING',
@@ -532,7 +578,8 @@ sub setup ($) {
 			     qw(always successful never));
 	    },
 	    DEFAULT => 'always',
-	    HELP => 'Purge the schroot session following a build.  This is useful in conjunction with the --purge and --purge-deps options when using snapshot chroots, since by default the snapshot will be deleted. Possible values are "always" (default), "never", and "successful"'
+	    HELP => 'Purge the schroot session following a build.  This is useful in conjunction with the --purge and --purge-deps options when using snapshot chroots, since by default the snapshot will be deleted. Possible values are "always" (default), "never", and "successful"',
+	    CLI_OPTIONS => ['-p', '--purge', '--purge-session']
 	},
 	'TOOLCHAIN_REGEX'			=> {
 	    TYPE => 'ARRAY:STRING',
@@ -584,10 +631,11 @@ sub setup ($) {
 
 		die "Bad chroot mode \'" . $conf->get('CHROOT_MODE') . "\'"
 		    if !isin($conf->get('CHROOT_MODE'),
-			     qw(schroot sudo));
+			     qw(schroot sudo adt));
 	    },
 	    DEFAULT => 'schroot',
-	    HELP => 'Mechanism to use for chroot virtualisation.  Possible value are "schroot" (default) and "sudo".'
+	    HELP => 'Mechanism to use for chroot virtualisation.  Possible value are "schroot" (default), "sudo" and "adt".',
+	    CLI_OPTIONS => ['--chroot-mode']
 	},
 	'CHROOT_SPLIT'				=> {
 	    TYPE => 'BOOL',
@@ -618,28 +666,32 @@ sub setup ($) {
 	    VARNAME => 'build_path',
 	    GROUP => 'Build options',
 	    DEFAULT => undef,
-	    HELP => 'By default the package is built in a path of the following format /build/packagename-XXXXXX/packagename-version/ where XXXXXX is a random ascii string. This option allows one to specify a custom path where the package is built inside the chroot. Notice that the sbuild user in the chroot must have permissions to create the path. Common writable locations are subdirectories of /tmp or /build. The buildpath must be an empty directory because the last component of the path will be removed after the build is finished. If you are running multiple sbuild instances with the same build path in parallel for the same package, make sure that your build path is not in a directory commonly mounted by all sbuild instances (like /tmp or /home). In that case, use for example /build instead. Otherwise, your builds will probably fail or contain wrong content.'
+	    HELP => 'By default the package is built in a path of the following format /build/packagename-XXXXXX/packagename-version/ where XXXXXX is a random ascii string. This option allows one to specify a custom path where the package is built inside the chroot. Notice that the sbuild user in the chroot must have permissions to create the path. Common writable locations are subdirectories of /tmp or /build. The buildpath must be an empty directory because the last component of the path will be removed after the build is finished. If you are running multiple sbuild instances with the same build path in parallel for the same package, make sure that your build path is not in a directory commonly mounted by all sbuild instances (like /tmp or /home). In that case, use for example /build instead. Otherwise, your builds will probably fail or contain wrong content.',
+	    CLI_OPTIONS => ['--build-path']
 	},
 	'SBUILD_MODE'				=> {
 	    TYPE => 'STRING',
 	    VARNAME => 'sbuild_mode',
 	    GROUP => 'Core options',
 	    DEFAULT => 'user',
-	    HELP => 'sbuild behaviour; possible values are "user" (exit status reports build failures) and "buildd" (exit status does not report build failures) for use in a buildd setup.  "buildd" also currently implies enabling of "legacy features" such as chroot symlinks in the build directory and the creation of current symlinks in the build directory.'
+	    HELP => 'sbuild behaviour; possible values are "user" (exit status reports build failures) and "buildd" (exit status does not report build failures) for use in a buildd setup.  "buildd" also currently implies enabling of "legacy features" such as chroot symlinks in the build directory and the creation of current symlinks in the build directory.',
+	    CLI_OPTIONS => ['--sbuild-mode']
 	},
 	'CHROOT_SETUP_SCRIPT'				=> {
 	    TYPE => 'STRING',
 	    VARNAME => 'chroot_setup_script',
 	    GROUP => 'Chroot options',
 	    DEFAULT => undef,
-	    HELP => 'Script to run to perform custom setup tasks in the chroot.'
+	    HELP => 'Script to run to perform custom setup tasks in the chroot.',
+	    CLI_OPTIONS => ['--setup-hook']
 	},
 	'FORCE_ORIG_SOURCE'			=> {
 	    TYPE => 'BOOL',
 	    VARNAME => 'force_orig_source',
 	    GROUP => 'Build options',
 	    DEFAULT => 0,
-	    HELP => 'By default, the -s option only includes the .orig.tar.gz when needed (i.e. when the Debian revision is 0 or 1).  By setting this option to 1, the .orig.tar.gz will always be included when -s is used.  This is equivalent to --force-orig-source.'
+	    HELP => 'By default, the -s option only includes the .orig.tar.gz when needed (i.e. when the Debian revision is 0 or 1).  By setting this option to 1, the .orig.tar.gz will always be included when -s is used.',
+	    CLI_OPTIONS => ['--force-orig-source']
 	},
 	'INDIVIDUAL_STALLED_PKG_TIMEOUT'	=> {
 	    TYPE => 'HASH:NUMERIC',
@@ -685,7 +737,8 @@ sub setup ($) {
 	    VARNAME => 'ld_library_path',
 	    GROUP => 'Build environment',
 	    DEFAULT => undef,
-	    HELP => 'Library search path to use inside the chroot.'
+	    HELP => 'Library search path to use inside the chroot.',
+	    CLI_OPTIONS => ['--use-snapshot']
 	},
 	'MAINTAINER_NAME'			=> {
 	    TYPE => 'STRING',
@@ -693,7 +746,8 @@ sub setup ($) {
 	    GROUP => 'Maintainer options',
 	    DEFAULT => undef,
 	    SET => $set_signing_option,
-	    HELP => 'Name to use as override in .changes files for the Maintainer field.  The Maintainer field will not be overridden unless set here.'
+	    HELP => 'Name to use as override in .changes files for the Maintainer field.  The Maintainer field will not be overridden unless set here.',
+	    CLI_OPTIONS => ['-m', '--maintainer']
 	},
 	'UPLOADER_NAME'				=> {
 	    VARNAME => 'uploader_name',
@@ -701,14 +755,16 @@ sub setup ($) {
 	    GROUP => 'Maintainer options',
 	    DEFAULT => undef,
 	    SET => $set_signing_option,
-	    HELP => 'Name to use as override in .changes file for the Changed-By: field.'
+	    HELP => 'Name to use as override in .changes file for the Changed-By: field.',
+	    CLI_OPTIONS => ['-e', '--uploader']
 	},
 	'KEY_ID'				=> {
 	    TYPE => 'STRING',
 	    VARNAME => 'key_id',
 	    GROUP => 'Maintainer options',
 	    DEFAULT => undef,
-	    HELP => 'Key ID to use in .changes for the current upload.  It overrides both $maintainer_name and $uploader_name.'
+	    HELP => 'Key ID to use in .changes for the current upload.  It overrides both $maintainer_name and $uploader_name.',
+	    CLI_OPTIONS => ['-k', '--keyid']
 	},
 	'SIGNING_OPTIONS'			=> {
 	    TYPE => 'STRING',
@@ -721,14 +777,16 @@ sub setup ($) {
 	    VARNAME => 'apt_clean',
 	    GROUP => 'Chroot options',
 	    DEFAULT => 0,
-	    HELP => 'APT clean.  1 to enable running "apt-get clean" at the start of each build, or 0 to disable.'
+	    HELP => 'APT clean.  1 to enable running "apt-get clean" at the start of each build, or 0 to disable.',
+	    CLI_OPTIONS => ['--apt-clean', '--no-apt-clean']
 	},
 	'APT_UPDATE'				=> {
 	    TYPE => 'BOOL',
 	    VARNAME => 'apt_update',
 	    GROUP => 'Chroot options',
 	    DEFAULT => 1,
-	    HELP => 'APT update.  1 to enable running "apt-get update" at the start of each build, or 0 to disable.'
+	    HELP => 'APT update.  1 to enable running "apt-get update" at the start of each build, or 0 to disable.',
+	    CLI_OPTIONS => ['--apt-update', '--no-apt-update']
 	},
 	'APT_UPDATE_ARCHIVE_ONLY'		=> {
 	    TYPE => 'BOOL',
@@ -742,14 +800,16 @@ sub setup ($) {
 	    VARNAME => 'apt_upgrade',
 	    GROUP => 'Chroot options',
 	    DEFAULT => 0,
-	    HELP => 'APT upgrade.  1 to enable running "apt-get upgrade" at the start of each build, or 0 to disable.'
+	    HELP => 'APT upgrade.  1 to enable running "apt-get upgrade" at the start of each build, or 0 to disable.',
+	    CLI_OPTIONS => ['--apt-upgrade', '--no-apt-upgrade']
 	},
 	'APT_DISTUPGRADE'			=> {
 	    TYPE => 'BOOL',
 	    VARNAME => 'apt_distupgrade',
 	    GROUP => 'Chroot options',
 	    DEFAULT => 1,
-	    HELP => 'APT distupgrade.  1 to enable running "apt-get dist-upgrade" at the start of each build, or 0 to disable.'
+	    HELP => 'APT distupgrade.  1 to enable running "apt-get dist-upgrade" at the start of each build, or 0 to disable.',
+	    CLI_OPTIONS => ['--apt-distupgrade', '--no-apt-distupgrade']
 	},
 	'APT_ALLOW_UNAUTHENTICATED'		=> {
 	    TYPE => 'BOOL',
@@ -760,9 +820,11 @@ sub setup ($) {
 	},
 	'BATCH_MODE'				=> {
 	    TYPE => 'BOOL',
-	    GROUP => '__INTERNAL',
+	    VARNAME => 'batch_mode',
+	    GROUP => 'Core options',
 	    DEFAULT => 0,
-	    HELP => 'Enable batch mode?'
+	    HELP => 'Enable batch mode?',
+	    CLI_OPTIONS => ['-b', '--batch']
 	},
 	'CORE_DEPENDS'				=> {
 	    TYPE => 'ARRAY:STRING',
@@ -773,39 +835,51 @@ sub setup ($) {
 	},
 	'MANUAL_DEPENDS'			=> {
 	    TYPE => 'ARRAY:STRING',
-	    GROUP => '__INTERNAL',
+	    VARNAME => 'manual_depends',
+	    GROUP => 'Core options',
 	    DEFAULT => [],
-	    HELP => 'Additional per-build dependencies.  Do not set by hand.'
+	    HELP => 'Additional per-build dependencies.',
+	    CLI_OPTIONS => ['--add-depends']
 	},
 	'MANUAL_CONFLICTS'			=> {
 	    TYPE => 'ARRAY:STRING',
-	    GROUP => '__INTERNAL',
+	    VARNAME => 'manual_conflicts',
+	    GROUP => 'Core options',
 	    DEFAULT => [],
-	    HELP => 'Additional per-build dependencies.  Do not set by hand.'
+	    HELP => 'Additional per-build dependencies.',
+	    CLI_OPTIONS => ['--add-conflicts']
 	},
 	'MANUAL_DEPENDS_ARCH'			=> {
 	    TYPE => 'ARRAY:STRING',
-	    GROUP => '__INTERNAL',
+	    VARNAME => 'manual_depends_arch',
+	    GROUP => 'Core options',
 	    DEFAULT => [],
-	    HELP => 'Additional per-build dependencies.  Do not set by hand.'
+	    HELP => 'Additional per-build dependencies.',
+	    CLI_OPTIONS => ['--add-depends-arch']
 	},
 	'MANUAL_CONFLICTS_ARCH'			=> {
 	    TYPE => 'ARRAY:STRING',
-	    GROUP => '__INTERNAL',
+	    VARNAME => 'manual_conflicts_arch',
+	    GROUP => 'Core options',
 	    DEFAULT => [],
-	    HELP => 'Additional per-build dependencies.  Do not set by hand.'
+	    HELP => 'Additional per-build dependencies.',
+	    CLI_OPTIONS => ['--add-conflicts-arch']
 	},
 	'MANUAL_DEPENDS_INDEP'			=> {
 	    TYPE => 'ARRAY:STRING',
-	    GROUP => '__INTERNAL',
+	    VARNAME => 'manual_depends_indep',
+	    GROUP => 'Core options',
 	    DEFAULT => [],
-	    HELP => 'Additional per-build dependencies.  Do not set by hand.'
+	    HELP => 'Additional per-build dependencies.',
+	    CLI_OPTIONS => ['--add-depends-indep']
 	},
 	'MANUAL_CONFLICTS_INDEP'		=> {
 	    TYPE => 'ARRAY:STRING',
-	    GROUP => '__INTERNAL',
+	    VARNAME => 'manual_conflicts_indep',
+	    GROUP => 'Core options',
 	    DEFAULT => [],
-	    HELP => 'Additional per-build dependencies.  Do not set by hand.'
+	    HELP => 'Additional per-build dependencies.',
+	    CLI_OPTIONS => ['--add-conflicts-indep']
 	},
 	'CROSSBUILD_CORE_DEPENDS'				=> {
 	    TYPE => 'HASH:ARRAY:STRING',
@@ -825,27 +899,33 @@ $crossbuild_core_depends = {
 	    GROUP => 'Build options',
 	    DEFAULT => 0,
 	    CHECK => $validate_append_version,
-	    HELP => 'By default, do not build a source package (binary only build).  Set to 1 to force creation of a source package, but note that this is inappropriate for binary NMUs, where the option will always be disabled. Set by the --source and --no-source command line options.'
+	    HELP => 'By default, do not build a source package (binary only build).  Set to 1 to force creation of a source package, but note that this is inappropriate for binary NMUs, where the option will always be disabled.',
+	    CLI_OPTIONS => ['-s', '--source', '--no-source']
 	},
 	'ARCHIVE'				=> {
 	    TYPE => 'STRING',
 	    VARNAME => 'archive',
 	    GROUP => 'Core options',
 	    DEFAULT => undef,
-	    HELP => 'Archive being built.  Only set in build log.  This might be useful for derivative distributions.'
+	    HELP => 'Archive being built.  Only set in build log.  This might be useful for derivative distributions.',
+	    CLI_OPTIONS => ['--archive']
 	},
 	'BIN_NMU'				=> {
 	    TYPE => 'STRING',
-	    GROUP => '__INTERNAL',
+	    VARNAME => 'bin_nmu',
+	    GROUP => 'Build options',
 	    DEFAULT => undef,
 	    CHECK => $validate_append_version,
-	    HELP => 'Binary NMU changelog entry.  Do not set by hand.'
+	    HELP => 'Binary NMU changelog entry.',
+	    CLI_OPTIONS => ['--make-binNMU']
 	},
 	'BIN_NMU_VERSION'			=> {
 	    TYPE => 'STRING',
-	    GROUP => '__INTERNAL',
+	    VARNAME => 'bin_nmu_version',
+	    GROUP => 'Build options',
 	    DEFAULT => undef,
-	    HELP => 'Binary NMU version number.  Do not set by hand.'
+	    HELP => 'Binary NMU version number.',
+	    CLI_OPTIONS => ['--binNMU', '--make-binNMU']
 	},
 	'APPEND_TO_VERSION'			=> {
 	    TYPE => 'STRING',
@@ -853,14 +933,16 @@ $crossbuild_core_depends = {
 	    GROUP => 'Build options',
 	    DEFAULT => undef,
 	    CHECK => $validate_append_version,
-	    HELP => 'Suffix to append to version number.  May be useful for derivative distributions.'
+	    HELP => 'Suffix to append to version number.  May be useful for derivative distributions.',
+	    CLI_OPTIONS => ['--append-to-version']
 	},
 	'GCC_SNAPSHOT'				=> {
 	    TYPE => 'BOOL',
 	    VARNAME => 'gcc_snapshot',
 	    GROUP => 'Build options',
 	    DEFAULT => 0,
-	    HELP => 'Build using current GCC snapshot?'
+	    HELP => 'Build using current GCC snapshot?',
+	    CLI_OPTIONS => ['--use-snapshot']
 	},
 	'JOB_FILE'				=> {
 	    TYPE => 'STRING',
@@ -890,14 +972,24 @@ $crossbuild_core_depends = {
 		    if !isin($conf->get($key),
 			     qw(apt aptitude aspcud xapt));
 	    },
-	    HELP => 'Build dependency resolver.  The \'apt\' resolver is currently the default, and recommended for most users.  This resolver uses apt-get to resolve dependencies.  Alternative resolvers are \'apt\', \'aptitude\' and \'aspcud\'. The \'apt\' resolver uses a built-in resolver module while the \'aptitude\' resolver uses aptitude to resolve build dependencies.  The aptitude resolver is similar to apt, but is useful in more complex situations, such as where multiple distributions are required, for example when building from experimental, where packages are needed from both unstable and experimental, but defaulting to unstable. If the dependency situation is too complex for either apt or aptitude to solve it, you can use the \'aspcud\' resolver which is a real SAT solver and will thus alwyas find a solution if a solution exists.'
+	    HELP => 'Build dependency resolver.  The \'apt\' resolver is currently the default, and recommended for most users.  This resolver uses apt-get to resolve dependencies.  Alternative resolvers are \'apt\', \'aptitude\' and \'aspcud\'. The \'apt\' resolver uses a built-in resolver module while the \'aptitude\' resolver uses aptitude to resolve build dependencies.  The aptitude resolver is similar to apt, but is useful in more complex situations, such as where multiple distributions are required, for example when building from experimental, where packages are needed from both unstable and experimental, but defaulting to unstable. If the dependency situation is too complex for either apt or aptitude to solve it, you can use the \'aspcud\' resolver which (in contrast to apt and aptitude) is a real solver (in the math sense) and will thus always find a solution if a solution exists.',
+	    CLI_OPTIONS => ['--build-dep-resolver']
+	},
+	'ASPCUD_CRITERIA'			=> {
+	    TYPE => 'STRING',
+	    VARNAME => 'aspcud_criteria',
+	    GROUP => 'Dependency resolution',
+	    DEFAULT => '-removed,-changed,-new',
+	    HELP => 'Optimization criteria in extended MISC 2012 syntax passed to aspcud through apt-cudf.  Optimization criteria are separated by commas, sorted by decreasing order of priority and are prefixed with a polarity (+ to maximize and - to minimize).  The default criteria is \'-removed,-changed,-new\' which first minimizes the number of removed packages, then the number of changed packages (up or downgrades) and then the number of new packages. A common task is to minimize the number of packages from experimental.  To do this you can add a criteria like \'-count(solution,APT-Release:=/a=experimental/)\' to the default criteria.  This will then minimize the number of packages in the solution which contain the string \'a=experimental\' in the \'APT-Release\' field of the EDSP output created by apt. See the apt-cudf man page help on the --criteria option for more information.',
+	    CLI_OPTIONS => ['--aspcud-criteria']
 	},
 	'CLEAN_SOURCE'				=> {
 	    TYPE => 'BOOL',
 	    VARNAME => 'clean_source',
 	    GROUP => 'Build options',
 	    DEFAULT => 1,
-	    HELP => 'When running sbuild from within an unpacked source tree, run the \'clean\' target before generating the source package. This might require some of the build dependencies necessary for running the \'clean\' target to be installed on the host machine. Only disable if you start from a clean checkout and you know what you are doing.'
+	    HELP => 'When running sbuild from within an unpacked source tree, run the \'clean\' target before generating the source package. This might require some of the build dependencies necessary for running the \'clean\' target to be installed on the host machine. Only disable if you start from a clean checkout and you know what you are doing.',
+	    CLI_OPTIONS => ['--clean-source', '--no-clean-source']
 	},
 	'LINTIAN'				=> {
 	    TYPE => 'STRING',
@@ -925,14 +1017,16 @@ $crossbuild_core_depends = {
 		$conf->check('LINTIAN');
 	    },
 	    DEFAULT => 0,
-	    HELP => 'Run lintian?'
+	    HELP => 'Run lintian?',
+	    CLI_OPTIONS => ['--run-lintian', '--no-run-lintian']
 	},
 	'LINTIAN_OPTIONS'			=> {
 	    TYPE => 'ARRAY:STRING',
 	    VARNAME => 'lintian_opts',
 	    GROUP => 'Build validation',
 	    DEFAULT => [],
-	    HELP => 'Options to pass to lintian.  Each option is a separate arrayref element.  For example, [\'-i\', \'-v\'] to add -i and -v.'
+	    HELP => 'Options to pass to lintian.  Each option is a separate arrayref element.  For example, [\'-i\', \'-v\'] to add -i and -v.',
+	    CLI_OPTIONS => ['--lintian-opt', '--lintian-opts']
 	},
 	'PIUPARTS'				=> {
 	    TYPE => 'STRING',
@@ -949,7 +1043,8 @@ $crossbuild_core_depends = {
 		}
 	    },
 	    DEFAULT => 'piuparts',
-	    HELP => 'Path to piuparts binary'
+	    HELP => 'Path to piuparts binary',
+	    CLI_OPTIONS => ['--piuparts-opt', '--piuparts-opts']
 	},
 	'RUN_PIUPARTS'				=> {
 	    TYPE => 'BOOL',
@@ -960,7 +1055,8 @@ $crossbuild_core_depends = {
 		$conf->check('PIUPARTS');
 	    },
 	    DEFAULT => 0,
-	    HELP => 'Run piuparts'
+	    HELP => 'Run piuparts',
+	    CLI_OPTIONS => ['--run-piuparts', '--no-run-piuparts']
 	},
 	'PIUPARTS_OPTIONS'			=> {
 	    TYPE => 'ARRAY:STRING',
@@ -974,7 +1070,8 @@ $crossbuild_core_depends = {
 	    VARNAME => 'piuparts_root_args',
 	    GROUP => 'Build validation',
 	    DEFAULT => [],
-	    HELP => 'Preceding arguments to launch piuparts as root. If no arguments are specified, piuparts will be launched via sudo.'
+	    HELP => 'Preceding arguments to launch piuparts as root. If no arguments are specified, piuparts will be launched via sudo.',
+	    CLI_OPTIONS => ['--piuparts-root-arg', '--piuparts-root-args']
 	},
 	'EXTERNAL_COMMANDS'			=> {
 	    TYPE => 'HASH:ARRAY:STRING',
@@ -983,6 +1080,7 @@ $crossbuild_core_depends = {
 	    DEFAULT => {
 		"pre-build-commands" => [],
 		"chroot-setup-commands" => [],
+		"chroot-update-failed-commands" => [],
 		"build-deps-failed-commands" => [],
 		"build-failed-commands" => [],
 		"starting-build-commands" => [],
@@ -1001,6 +1099,10 @@ $crossbuild_core_depends = {
         [\'foo\', \'arg1\', \'arg2\'],
         [\'bar\', \'arg1\', \'arg2\', \'arg3\'],
     ],
+    "chroot-update-failed-commands" => [
+        [\'foo\', \'arg1\', \'arg2\'],
+        [\'bar\', \'arg1\', \'arg2\', \'arg3\'],
+    ],
     "build-deps-failed-commands" => [
         [\'foo\', \'arg1\', \'arg2\'],
         [\'bar\', \'arg1\', \'arg2\', \'arg3\'],
@@ -1025,21 +1127,24 @@ $crossbuild_core_depends = {
         [\'foo\', \'arg1\', \'arg2\'],
         [\'bar\', \'arg1\', \'arg2\', \'arg3\'],
     ],
-};'
+};',
+	    CLI_OPTIONS => ['--setup-hook', '--pre-build-commands', '--chroot-setup-commands', '--chroot-update-failed-commands', '--build-deps-failed-commands', '--build-failed-commands', '--anything-failed-commands', '--starting-build-commands', '--finished-build-commands', '--chroot-cleanup-commands', '--post-build-commands']
 	},
 	'LOG_EXTERNAL_COMMAND_OUTPUT'		=> {
 	    TYPE => 'BOOL',
 	    VARNAME => 'log_external_command_output',
 	    GROUP => 'Chroot options',
 	    DEFAULT => 1,
-	    HELP => 'Log standard output of commands run by sbuild?'
+	    HELP => 'Log standard output of commands run by sbuild?',
+	    CLI_OPTIONS => ['--log-external-command-output']
 	},
 	'LOG_EXTERNAL_COMMAND_ERROR'		=> {
 	    TYPE => 'BOOL',
 	    VARNAME => 'log_external_command_error',
 	    GROUP => 'Chroot options',
 	    DEFAULT => 1,
-	    HELP => 'Log standard error of commands run by sbuild?'
+	    HELP => 'Log standard error of commands run by sbuild?',
+	    CLI_OPTIONS => ['--log-external-command-error']
 	},
 	'RESOLVE_ALTERNATIVES'				=> {
 	    TYPE => 'BOOL',
@@ -1061,7 +1166,8 @@ $crossbuild_core_depends = {
 		return $retval;
 	    },
 	    EXAMPLE => '$resolve_alternatives = 0;',
-	    HELP => 'Should the dependency resolver use alternatives in Build-Depends, Build-Depends-Arch and Build-Depends-Indep?  By default, using \'apt\' resolver, only the first alternative will be used; all other alternatives will be removed.  When using the \'aptitude\' resolver, it will default to using all alternatives.  Note that this does not include architecture-specific alternatives, which are reduced to the build architecture prior to alternatives removal.  This should be left disabled when building for unstable; it may be useful when building for experimental or backports.  Set to undef to use the default, 1 to enable, or 0 to disable.'
+	    HELP => 'Should the dependency resolver use alternatives in Build-Depends, Build-Depends-Arch and Build-Depends-Indep?  By default, using \'apt\' resolver, only the first alternative will be used; all other alternatives will be removed.  When using the \'aptitude\' resolver, it will default to using all alternatives.  Note that this does not include architecture-specific alternatives, which are reduced to the build architecture prior to alternatives removal.  This should be left disabled when building for unstable; it may be useful when building for experimental or backports.  Set to undef to use the default, 1 to enable, or 0 to disable.',
+	    CLI_OPTIONS => ['--resolve-alternatives', '--no-resolve-alternatives']
 	},
 	'SBUILD_BUILD_DEPENDS_SECRET_KEY'		=> {
 	    TYPE => 'STRING',
@@ -1079,21 +1185,27 @@ $crossbuild_core_depends = {
 	},
 	'EXTRA_PACKAGES'				=> {
 	    TYPE => 'ARRAY:STRING',
-	    GROUP => '__INTERNAL',
+	    VARNAME => 'extra_packages',
+	    GROUP => 'Dependency resolution',
 	    DEFAULT => [],
-	    HELP => 'Additional per-build packages available as build dependencies.  Do not set by hand.'
+	    HELP => 'Additional per-build packages available as build dependencies.',
+	    CLI_OPTIONS => ['--extra-package']
 	},
 	'EXTRA_REPOSITORY_KEYS'				=> {
 	    TYPE => 'ARRAY:STRING',
-	    GROUP => '__INTERNAL',
+	    VARNAME => 'extra_repository_keys',
+	    GROUP => 'Dependency resolution',
 	    DEFAULT => [],
-	    HELP => 'Additional per-build apt repository keys.  Do not set by hand.'
+	    HELP => 'Additional per-build apt repository keys.',
+	    CLI_OPTIONS => ['--extra-repository-key']
 	},
 	'EXTRA_REPOSITORIES'				=> {
 	    TYPE => 'ARRAY:STRING',
-	    GROUP => '__INTERNAL',
+	    VARNAME => 'extra_repositories',
+	    GROUP => 'Dependency resolution',
 	    DEFAULT => [],
-	    HELP => 'Additional per-build apt repositories.  Do not set by hand.'
+	    HELP => 'Additional per-build apt repositories.',
+	    CLI_OPTIONS => ['--extra-repository']
 	},
     );
 
